@@ -1,19 +1,17 @@
 use strict;
 use warnings;
 package Dist::Zilla::Plugin::CheckIssues;
-BEGIN {
-  $Dist::Zilla::Plugin::CheckIssues::AUTHORITY = 'cpan:ETHER';
-}
-# git description: v0.001-5-g4b12dc9
-$Dist::Zilla::Plugin::CheckIssues::VERSION = '0.002';
-# ABSTRACT: Retrieve count of outstanding RT and github issues
+# git description: v0.002-17-g6a9cdf8
+$Dist::Zilla::Plugin::CheckIssues::VERSION = '0.003';
+# ABSTRACT: Retrieve count of outstanding RT and github issues for your distribution
+# KEYWORDS: plugin bugs issues rt github
 # vim: set ts=8 sw=4 tw=78 et :
 
 use Moose;
 with 'Dist::Zilla::Role::BeforeRelease';
-use HTTP::Tiny;
-use JSON::MaybeXS;
+use List::Util 1.33 'none';
 use Term::ANSIColor 'colored';
+use Encode ();
 use namespace::autoclean;
 
 has [qw(rt github colour)] => (
@@ -21,51 +19,81 @@ has [qw(rt github colour)] => (
     default => 1,
 );
 
-# [ user/org name, repo name ]
-has _github_repository => (
-    isa => 'ArrayRef[Str]',
+has repo_url => (
+    is => 'rw', isa => 'Str',
     lazy => 1,
     default => sub {
         my $self = shift;
 
-        my $distmeta = $self->zilla->distmeta;
-        my $url = (($distmeta->{resources} || {})->{repository} || {})->{url} || '';
+        my $url;
+        if ($self->zilla->built_in)
+        {
+            # we've already done a build, so distmeta is available
+            my $distmeta = $self->zilla->distmeta;
+            $url = (($distmeta->{resources} || {})->{repository} || {})->{url} || '';
+        }
+        else
+        {
+            # no build (we're probably running the command): we cannot simply
+            # call ->distmeta because various plugins will cause side-effects
+            # with invalid assumptions (no files have been gathered, etc) --
+            # so we just query a short list of plugins that we know can
+            # provide repository resource metadata
+            foreach my $plugin (@{ $self->zilla->plugins_with(-MetaProvider) })
+            {
+                next if none { $plugin->isa('Dist::Zilla::Plugin::' . $_) }
+                    qw(MetaResources AutoMetaResources GithubMeta GitHub::Meta Repository);
 
-        my ($org_name, $repo_name) = $url =~ m{github\.com/([^/]+)/([^/]+?)(?:/|\.git|$)};
+                $self->log_debug('calling metadata for ' . $plugin->plugin_name);
+                my $plugin_meta = $plugin->metadata;
+                $url = (($plugin_meta->{resources} || {})->{repository} || {})->{url} || '';
+                last if $url;
+            }
+        }
+        $url;
+    },
+);
 
-        return [ $org_name, $repo_name ] if $org_name and $repo_name;
+has _github_owner_repo => (
+    isa => 'ArrayRef[Str]',
+    init_arg => undef,
+    lazy => 1,
+    default => sub {
+        my $self = shift;
+
+        if (my $url = $self->repo_url)
+        {
+            $self->log_debug('getting issue data for ' . $url . '...');
+            my ($owner_name, $repo_name) = $url =~ m{github\.com[:/]([^/]+)/([^/]+?)(?:/|\.git|$)};
+            return [ $owner_name, $repo_name ] if $owner_name and $repo_name;
+        }
 
         $self->log('failed to find a github repo in metadata');
         [];
     },
     traits => ['Array'],
-    handles => { _github_repository => 'elements' },
-);
-
-has repo_url => (
-    is => 'ro', isa => 'Str',
-    lazy => 1,
-    default => sub {
-        my $self = shift;
-        my ($org_name, $repo_name) = $self->_github_repository;
-        return "https://github.com/$org_name/$repo_name" if $org_name and $repo_name;
-        '';
-    },
+    handles => { _github_owner_repo => 'elements' },
 );
 
 sub mvp_aliases { +{ color => 'colour' } }
 
-sub before_release
+# metaconfig is unimportant for this distribution since it does not alter the
+# built distribution in any way
+# around dump_config => sub...
+
+sub get_issues
 {
     my $self = shift;
 
     my $dist_name = $self->zilla->name;
 
+    my @issues;
+
     if ($self->rt)
     {
         my %rt_data = $self->_rt_data_for_dist($dist_name);
 
-        my $colour = $rt_data{open} ? 'red'
+        my $colour = $rt_data{open} ? 'orange'
             : $rt_data{stalled} ? 'yellow'
             : 'green';
 
@@ -75,28 +103,35 @@ sub before_release
         );
 
         @text = map { colored($_, $colour) } @text if $self->colour;
-        $self->log($_) foreach @text;
+        push @issues, @text;
     }
 
     if ($self->github
-        and my ($owner_name, $repo_name) = $self->_github_repository)
+        and my ($owner_name, $repo_name) = $self->_github_owner_repo)
     {
         my $issue_count = $self->_github_issue_count($owner_name, $repo_name);
         if (defined $issue_count)
         {
-            my $colour = $issue_count ? 'red' : 'green';
+            my $colour = $issue_count ? 'orange' : 'green';
 
             my @text = (
-                'Issues on github (' . $self->repo_url . '):',
+                'Issues on github (https://github.com/' . $owner_name . '/' . $repo_name . '):',
                 '  open: ' . $issue_count,
             );
 
             @text = map { colored($_, $colour) } @text if $self->colour;
-            $self->log($_) foreach @text;
+            push @issues, @text;
         }
     }
 
-    return;
+    return @issues;
+}
+
+sub before_release
+{
+    my $self = shift;
+
+    $self->log($_) foreach $self->get_issues;
 }
 
 sub _rt_data_for_dist
@@ -106,7 +141,8 @@ sub _rt_data_for_dist
     my $json = $self->_rt_data_raw;
     return if not $json;
 
-    my $all_data = decode_json($json);
+    require JSON::MaybeXS;
+    my $all_data = JSON::MaybeXS->new(utf8 => 0)->decode($json);
     return if not $all_data->{$dist_name};
 
     my %rt_data;
@@ -121,9 +157,9 @@ sub _rt_data_raw
     my $self = shift;
 
     $self->log_debug('fetching RT bug data...');
-    my $res = HTTP::Tiny->new->get('https://rt.cpan.org/Public/bugs-per-dist.json');
-    $self->log('could not fetch RT data?'), return if not $res->{success};
-    return $res->{content};
+    my $data = $self->_fetch('https://rt.cpan.org/Public/bugs-per-dist.json');
+    $self->log('could not fetch RT data?'), return if not $data;
+    return $data;
 }
 
 sub _github_issue_count
@@ -131,14 +167,34 @@ sub _github_issue_count
     my ($self, $owner_name, $repo_name) = @_;
 
     $self->log_debug('fetching github issues data...');
-    my $res = HTTP::Tiny->new->get('https://api.github.com/repos/' . $owner_name . '/' . $repo_name);
-    $self->log('could not fetch github data?'), return if not $res->{success};
-    my $json = $res->{content};
 
-    my $data = decode_json($json);
+    my $json = $self->_fetch('https://api.github.com/repos/' . $owner_name . '/' . $repo_name);
+    $self->log('could not fetch github data?'), return if not $json;
+
+    require JSON::MaybeXS;
+    my $data = JSON::MaybeXS->new(utf8 => 0)->decode($json);
     $data->{open_issues_count};
 }
 
+sub _fetch
+{
+    my ($self, $url) = @_;
+
+    require HTTP::Tiny;
+    my $res = HTTP::Tiny->new->get($url);
+    return if not $res->{success};
+
+    my $data = $res->{content};
+
+    require HTTP::Headers;
+    if (my $charset = HTTP::Headers->new(%{ $res->{headers} })->content_type_charset)
+    {
+        $data = Encode::decode($charset, $data, Encode::FB_CROAK);
+    }
+
+    return $data;
+}
+
 __PACKAGE__->meta->make_immutable;
 
 __END__
@@ -149,11 +205,11 @@ __END__
 
 =head1 NAME
 
-Dist::Zilla::Plugin::CheckIssues - Retrieve count of outstanding RT and github issues
+Dist::Zilla::Plugin::CheckIssues - Retrieve count of outstanding RT and github issues for your distribution
 
 =head1 VERSION
 
-version 0.002
+version 0.003
 
 =head1 SYNOPSIS
 
@@ -173,7 +229,7 @@ counts for your distribution before release.  Place it immediately before
 C<[ConfirmRelease]> in your F<dist.ini> to give you an opportunity to abort the
 release if you forgot to fix a bug or merge a pull request.
 
-=for Pod::Coverage mvp_aliases before_release
+=for Pod::Coverage mvp_aliases before_release get_issues
 
 =head1 CONFIGURATION OPTIONS
 
@@ -189,13 +245,16 @@ Checks the issue list on L<github|https://github.com> for your distribution; doe
 nothing if your distribution is not hosted on L<github|https://github.com>, as
 listed in your distribution's metadata.  Defaults to true.
 
-(Not yet implemented. Coming soon!)
-
-=head2 C<color> or C<colour>
+=head2 C<colour> or C<color>
 
 Uses L<Term::ANSIColor> to colour-code the results according to severity.
 Defaults to true.
 
+=head2 C<repo_url>
+
+The URL of the github repository.  This is fetched from the C<resources> field
+in metadata, so it should not normally be specified manually.
+
 =head1 FUTURE FEATURES, MAYBE
 
 If I can find the right APIs to call, it would be nice to have a C<verbose>
@@ -231,6 +290,10 @@ L<Dist::Zilla::Plugin::MetaResources> - manually add resource information (such
 
 L<Dist::Zilla::Plugin::GithubMeta> - automatically detect and add github repository information to metadata
 
+=item *
+
+L<Dist::Zilla::Plugin::AutoMetaResources> - configuration-based resource metadata provider
+
 =back
 
 =head1 AUTHOR
